/*
 * ClockManager.java
 *
 * Copyright (c) 2001- Kenta Cho. All rights reserved.
 */

import com.nttdocomo.ui.*;
import com.nttdocomo.io.ConnectionException;
import java.util.Calendar;
import java.util.Random;

/**
 * Clock management thread.
 *
 * @author cs8k-cyu@asahi-net.or.jp
 */
public class ClockManager {
  private static final int HAND_NUM = 3;
  private Hand[] hand = new Hand[HAND_NUM];

  private int ptpTime;

  private static Random rnd = new Random();

  private static final int INTERVAL= 100;
  private static final int MAX_FRAME_SKIP = 8;

  private Graphics grp;
  private int scWidth, scHeight, scCenterX, scCenterY;
  private int frontColor;

  private FlpCanvas canvas;

  private int randN(int n) {
    return Math.abs(rnd.nextInt()%n);
  }

  public ClockManager() {
    loadImages();
  }

  private boolean backLight = true;

  public void setBackLight() {
    int blStat;
    if ( backLight ) {
      blStat = PhoneSystem.ATTR_BACKLIGHT_ON;
    } else {
      blStat = PhoneSystem.ATTR_BACKLIGHT_OFF;
    }
    PhoneSystem.setAttribute(PhoneSystem.DEV_BACKLIGHT, blStat);
  }

  public void init(FlpCanvas canvas) {
    this.canvas = canvas;
    grp = canvas.getGraphics();
    scWidth = canvas.getWidth();
    scHeight = canvas.getHeight();
    Hand.xClockCenter = scCenterX = scWidth/2;
    Hand.yClockCenter = scCenterY = scHeight/2;
    if ( scWidth > scHeight ) {
      Hand.lengthAdjust = scHeight;
    } else {
      Hand.lengthAdjust = scWidth;
    }
    hand[0] = new Hand(50, 15, grp);
    hand[1] = new Hand(90, 8, grp);
    hand[2] = new Hand(75, 0, grp);
    setTime();
    for ( int i=0 ; i<HAND_NUM ; i++ ) {
      hand[i].deg = hand[i].amDeg;
    }
    ptpTime = randN(400)+400;
    frontColor = Graphics.getColorOfName(Graphics.BLACK);
    setBackLight();
  }

  private Image[] letters = new Image[12];
  private String[] rscNames = {
    "iti.gif", "ni.gif", "san.gif", "si.gif", "go.gif", "roku.gif",
    "siti.gif", "hati.gif", "kyuu.gif", "zyuu.gif", "zyuuiti.gif", "zyuuni.gif"
  };

  private void loadImages() {
    MediaImage media;
    for ( int i=0 ; i<rscNames.length ; i++ ) {
      try {
        media = MediaManager.getImage("resource:///" + rscNames[i]);
        media.use();
        letters[i] = media.getImage();
      } catch (ConnectionException e) {}
    }
  }

  private void drawLetters() {
    grp.drawImage(letters[0], scCenterX+scWidth/4-4, 0);
    grp.drawImage(letters[1], scWidth-9, scCenterY-scHeight/4-4);
    grp.drawImage(letters[2], scWidth-17, scCenterY-4);
    grp.drawImage(letters[3], scWidth-9, scCenterY+scHeight/4-4);
    grp.drawImage(letters[4], scCenterX+scWidth/4-4, scHeight-9);
    grp.drawImage(letters[5], scCenterX-8, scHeight-16);
    grp.drawImage(letters[6], scCenterX-scWidth/4-4, scHeight-9);
    grp.drawImage(letters[7], 0, scCenterY+scHeight/4-4);
    grp.drawImage(letters[8], 0, scCenterY-4);
    grp.drawImage(letters[9], 0, scCenterY-scHeight/4-4);
    grp.drawImage(letters[10], scCenterX-scWidth/4-4, 0);
    grp.drawImage(letters[11], scCenterX-12, 0);
  }

  private static int clrSize = 0;

  public static void setClearRectSize(int sz) {
    if ( sz > clrSize ) {
      clrSize = sz;
    }
  }

  /**
   * set current time(hour/minute/second)
   */
  private void setTime() {
    Calendar calendar = Calendar.getInstance();
    int hr = calendar.get(Calendar.HOUR),
        mn = calendar.get(Calendar.MINUTE),
        sc = calendar.get(Calendar.SECOND);
    hand[0].amDeg = hr*5*4 + mn/3;
    if ( hand[0].amDeg >= 240 ) hand[0].amDeg -= 240;
    hand[1].amDeg = mn*4 + sc/15;
    hand[2].amDeg = sc*4;
  }

  /**
   * set current second
   */
  private void setSecond() {
    Calendar calendar = Calendar.getInstance();
    hand[2].amDeg = calendar.get(Calendar.SECOND)*4;
  }

  /**
   * try to show accurate time
   */
  public void addPower() {
    setTime();
    for ( int i=0 ; i<HAND_NUM ; i++ ) {
      hand[i].addPower();
    }
    ptpTime = randN(400)+400;
  }

  private boolean pPrsd = true;

  public void loop() {
    long prvTicks, nowTicks;
    int frame;

    grp.lock();
    grp.clearRect(0, 0, scWidth, scHeight);
    drawLetters();
    grp.unlock(true);

    prvTicks = System.currentTimeMillis();
    for ( ; ; ) {
      nowTicks = System.currentTimeMillis();
      frame = (int)(nowTicks-prvTicks)/INTERVAL;
      if ( frame <= 0 ) {
        frame = 1;
        prvTicks += INTERVAL;
        while ( System.currentTimeMillis() < prvTicks ) {}
      } else {
        prvTicks = nowTicks;
        //prvTicks += frame*INTERVAL;
        if ( frame > MAX_FRAME_SKIP ) frame = MAX_FRAME_SKIP;
      }
      for ( ; frame>0 ; frame-- ) {
        for ( int i=0 ; i<HAND_NUM ; i++ ) {
          hand[i].moveHand();
        }
        if ( (hand[2].ptPower%10) == 9 ) {
          setSecond();
        } else if ( (hand[2].ptPower%10) == -1 ) {
          hand[2].deg += 4;
          if ( hand[2].deg >= 240 ) hand[2].deg -= 240;
        }
        ptpTime--;
        if ( ptpTime <= 0 ) {
          addPower();
        }
      }
      grp.lock();
      grp.clearRect(scCenterX-clrSize, scCenterY-clrSize, clrSize*2, clrSize*2);
      grp.setColor(frontColor);
      for ( int i=0 ; i<HAND_NUM ; i++ ) {
        hand[i].draw();
      }
      grp.unlock(true);
      int key = canvas.getKeypadState();
      if ( (key&(1<<Display.KEY_SOFT1)) != 0 ) {
        addPower();
      }
      if ( (key&(1<<Display.KEY_SOFT2)) != 0 ) {
        IApplication.getCurrentApp().terminate();
      }
      if ( (key&(1<<Display.KEY_POUND)) == 0 ) {
        pPrsd = false;
      } else if ( !pPrsd ) {
        pPrsd = true;
        if ( backLight ) {
          backLight = false;
        } else {
          backLight = true;
        }
        setBackLight();
      }
    }
  }
}
